@@ -80,7 +80,7 @@ static int	hash_cb_strict(VALUE key, VALUE value, Out out);
 static int	hash_cb_compat(VALUE key, VALUE value, Out out);
 static int	hash_cb_object(VALUE key, VALUE value, Out out);
 static void	dump_hash(VALUE obj, VALUE clas, int depth, int mode, Out out);
-static void	dump_time(VALUE obj, Out out);
+static void	dump_time(VALUE obj, Out out, int withZone);
 static void	dump_ruby_time(VALUE obj, Out out);
 static void	dump_xml_time(VALUE obj, Out out);
 static void	dump_data_strict(VALUE obj, Out out);
@@ -399,7 +399,7 @@ static void
 dump_fixnum(VALUE obj, Out out) {
     char	buf[32];
     char	*b = buf + sizeof(buf) - 1;
-    long	num = NUM2LONG(obj);
+    long long	num = rb_num2ll(obj);
     int		neg = 0;
 
     if (0 > num) {
@@ -457,18 +457,57 @@ dump_float(VALUE obj, Out out) {
 	*b++ = '\0';
 	cnt = 3;
     } else if (OJ_INFINITY == d) {
-	strcpy(buf, "Infinity");
-	cnt = 8;
+	switch (out->opts->mode) {
+	case StrictMode:
+	    raise_strict(obj);
+	case NullMode:
+	    strcpy(buf, "null");
+	    cnt = 4;
+	    break;
+	default:
+	    strcpy(buf, "Infinity");
+	    cnt = 8;
+	    break;
+	}
     } else if (-OJ_INFINITY == d) {
-	strcpy(buf, "-Infinity");
-	cnt = 9;
+	switch (out->opts->mode) {
+	case StrictMode:
+	    raise_strict(obj);
+	case NullMode:
+	    strcpy(buf, "null");
+	    cnt = 4;
+	    break;
+	default:
+	    strcpy(buf, "-Infinity");
+	    cnt = 9;
+	    break;
+	}
     } else if (isnan(d)) {
-	strcpy(buf, "NaN");
-	cnt = 3;
+	switch (out->opts->mode) {
+	case StrictMode:
+	    raise_strict(obj);
+	case NullMode:
+	    strcpy(buf, "null");
+	    cnt = 4;
+	    break;
+	default:
+	    strcpy(buf, "NaN");
+	    cnt = 3;
+	    break;
+	}
     } else if (d == (double)(long long int)d) {
-	cnt = sprintf(buf, "%.1f", d); // used sprintf due to bug in snprintf
+	cnt = snprintf(buf, sizeof(buf), "%.1f", d);
+    } else if (0 == out->opts->float_prec) {
+	volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
+
+	cnt = RSTRING_LEN(rstr);
+	if ((int)sizeof(buf) <= cnt) {
+	    cnt = sizeof(buf) - 1;
+	}
+	strncpy(buf, rb_string_value_ptr((VALUE*)&rstr), cnt);
+	buf[cnt] = '\0';
     } else {
-	cnt = sprintf(buf, "%0.16g", d); // used sprintf due to bug in snprintf
+	cnt = snprintf(buf, sizeof(buf), out->opts->float_fmt, d);
     }
     if (out->end - out->cur <= (long)cnt) {
 	grow(out, cnt);
@@ -636,7 +675,7 @@ dump_array(VALUE a, VALUE clas, int depth, Out out) {
     if (id < 0) {
 	return;
     }
-    if (Qundef != clas && rb_cArray != clas) {
+    if (Qundef != clas && rb_cArray != clas && ObjectMode == out->opts->mode) {
 	dump_obj_attrs(a, clas, 0, depth, out);
 	return;
     }
@@ -978,11 +1017,11 @@ dump_hash(VALUE obj, VALUE clas, int depth, int mode, Out out) {
 }
 
 static void
-dump_time(VALUE obj, Out out) {
+dump_time(VALUE obj, Out out, int withZone) {
     char		buf[64];
     char		*b = buf + sizeof(buf) - 1;
     long		size;
-    char		*dot = b - 10;
+    char		*dot;
     int			neg = 0;
     long		one = 1000000000;
 #if HAS_RB_TIME_TIMESPEC
@@ -992,12 +1031,35 @@ dump_time(VALUE obj, Out out) {
 #else
     time_t		sec = NUM2LONG(rb_funcall2(obj, oj_tv_sec_id, 0, 0));
 #if HAS_NANO_TIME
-    long		nsec = NUM2LONG(rb_funcall2(obj, oj_tv_nsec_id, 0, 0));
+    long long		nsec = rb_num2ll(rb_funcall2(obj, oj_tv_nsec_id, 0, 0));
 #else
-    long		nsec = NUM2LONG(rb_funcall2(obj, oj_tv_usec_id, 0, 0)) * 1000;
+    long long		nsec = rb_num2ll(rb_funcall2(obj, oj_tv_usec_id, 0, 0)) * 1000;
 #endif
 #endif
     
+    *b-- = '\0';
+    if (withZone) {
+	long	tzsecs = NUM2LONG(rb_funcall2(obj, oj_utc_offset_id, 0, 0));
+	int	zneg = (0 > tzsecs);
+
+	if (0 == tzsecs && Qtrue == rb_funcall2(obj, oj_utcq_id, 0, 0)) {
+	    tzsecs = 86400;
+	}
+	if (zneg) {
+	    tzsecs = -tzsecs;
+	}
+	if (0 == tzsecs) {
+	    *b-- = '0';
+	} else {
+	    for (; 0 < tzsecs; b--, tzsecs /= 10) {
+		*b = '0' + (tzsecs % 10);
+	    }
+	    if (zneg) {
+		*b-- = '-';
+	    }
+	}
+	*b-- = 'e';
+    }
     if (0 > sec) {
 	neg = 1;
 	sec = -sec;
@@ -1006,7 +1068,7 @@ dump_time(VALUE obj, Out out) {
 	    sec--;
 	}
     }
-    *b-- = '\0';
+    dot = b - 9;
     if (0 < out->opts->sec_prec) {
 	if (9 > out->opts->sec_prec) {
 	    int	i;
@@ -1065,12 +1127,12 @@ dump_xml_time(VALUE obj, Out out) {
 #else
     time_t		sec = NUM2LONG(rb_funcall2(obj, oj_tv_sec_id, 0, 0));
 #if HAS_NANO_TIME
-    long		nsec = NUM2LONG(rb_funcall2(obj, oj_tv_nsec_id, 0, 0));
+    long long		nsec = rb_num2ll(rb_funcall2(obj, oj_tv_nsec_id, 0, 0));
 #else
-    long		nsec = NUM2LONG(rb_funcall2(obj, oj_tv_usec_id, 0, 0)) * 1000;
+    long long		nsec = rb_num2ll(rb_funcall2(obj, oj_tv_usec_id, 0, 0)) * 1000;
 #endif
 #endif
-    long		tz_secs = NUM2LONG(rb_funcall2(obj, oj_utc_offset_id, 0, 0));
+    long		tzsecs = NUM2LONG(rb_funcall2(obj, oj_utc_offset_id, 0, 0));
     int			tzhour, tzmin;
     char		tzsign = '+';
 
@@ -1091,16 +1153,16 @@ dump_xml_time(VALUE obj, Out out) {
     }
     // 2012-01-05T23:58:07.123456000+09:00
     //tm = localtime(&sec);
-    sec += tz_secs;
+    sec += tzsecs;
     tm = gmtime(&sec);
 #if 1
-    if (0 > tz_secs) {
+    if (0 > tzsecs) {
         tzsign = '-';
-        tzhour = (int)(tz_secs / -3600);
-        tzmin = (int)(tz_secs / -60) - (tzhour * 60);
+        tzhour = (int)(tzsecs / -3600);
+        tzmin = (int)(tzsecs / -60) - (tzhour * 60);
     } else {
-        tzhour = (int)(tz_secs / 3600);
-        tzmin = (int)(tz_secs / 60) - (tzhour * 60);
+        tzhour = (int)(tzsecs / 3600);
+        tzmin = (int)(tzsecs / 60) - (tzhour * 60);
     }
 #else
     if (0 > tm->tm_gmtoff) {
@@ -1113,7 +1175,7 @@ dump_xml_time(VALUE obj, Out out) {
     }
 #endif
     if (0 == nsec || 0 == out->opts->sec_prec) {
-	if (0 == tzhour && 0 == tzmin) {
+	if (0 == tzsecs && Qtrue == rb_funcall2(obj, oj_utcq_id, 0, 0)) {
 	    sprintf(buf, "%04d-%02d-%02dT%02d:%02d:%02dZ",
 		    tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
 		    tm->tm_hour, tm->tm_min, tm->tm_sec);
@@ -1125,6 +1187,18 @@ dump_xml_time(VALUE obj, Out out) {
 		    tzsign, tzhour, tzmin);
 	    dump_cstr(buf, 25, 0, 0, out);
 	}
+    } else if (0 == tzsecs && Qtrue == rb_funcall2(obj, oj_utcq_id, 0, 0)) {
+	char	format[64] = "%04d-%02d-%02dT%02d:%02d:%02d.%09ldZ";
+	int	len = 30;
+
+	if (9 > out->opts->sec_prec) {
+	    format[32] = '0' + out->opts->sec_prec;
+	    len -= 9 - out->opts->sec_prec;
+	}
+	sprintf(buf, format,
+		tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+		tm->tm_hour, tm->tm_min, tm->tm_sec, nsec);
+	dump_cstr(buf, len, 0, 0, out);
     } else {
 	char	format[64] = "%04d-%02d-%02dT%02d:%02d:%02d.%09ld%c%02d:%02d";
 	int	len = 35;
@@ -1169,7 +1243,7 @@ dump_data_null(VALUE obj, Out out) {
 
 static void
 dump_data_comp(VALUE obj, int depth, Out out) {
-    volatile VALUE	o2;
+    VALUE	clas = rb_obj_class(obj);
 
     if (rb_respond_to(obj, oj_to_hash_id)) {
 	volatile VALUE	h = rb_funcall(obj, oj_to_hash_id, 0);
@@ -1178,8 +1252,22 @@ dump_data_comp(VALUE obj, int depth, Out out) {
 	    rb_raise(rb_eTypeError, "%s.to_hash() did not return a Hash.\n", rb_class2name(rb_obj_class(obj)));
 	}
 	dump_hash(h, Qundef, depth, out->opts->mode, out);
-    } else if (rb_respond_to(obj, oj_as_json_id) && obj != (o2 = rb_funcall(obj, oj_as_json_id, 0))) {
-	dump_val(o2, depth, out);
+
+    } else if (Yes == out->opts->bigdec_as_num && oj_bigdecimal_class == clas) {
+	volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
+
+	dump_raw(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), out);
+    } else if (rb_respond_to(obj, oj_as_json_id)) {
+	volatile VALUE	aj = rb_funcall(obj, oj_as_json_id, 0);
+
+	// Catch the obvious brain damaged recursive dumping.
+	if (aj == obj) {
+	    volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
+
+	    dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
+	} else {
+	    dump_val(aj, depth, out);
+	}
     } else if (Yes == out->opts->to_json && rb_respond_to(obj, oj_to_json_id)) {
 	volatile VALUE	rs;
 	const char	*s;
@@ -1196,23 +1284,18 @@ dump_data_comp(VALUE obj, int depth, Out out) {
 	out->cur += len;
 	*out->cur = '\0';
     } else {
-	VALUE	clas = rb_obj_class(obj);
-
 	if (rb_cTime == clas) {
 	    switch (out->opts->time_format) {
 	    case RubyTime:	dump_ruby_time(obj, out);	break;
 	    case XmlTime:	dump_xml_time(obj, out);	break;
+	    case UnixZTime:	dump_time(obj, out, 1);		break;
 	    case UnixTime:
-	    default:		dump_time(obj, out);		break;
+	    default:		dump_time(obj, out, 0);		break;
 	    }
 	} else if (oj_bigdecimal_class == clas) {
 	    volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
 
-	    if (Yes == out->opts->bigdec_as_num) {
-		dump_raw(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), out);
-	    } else {
-		dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
-	    }
+	    dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
 	} else {
 	    volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
 
@@ -1240,9 +1323,12 @@ dump_data_obj(VALUE obj, int depth, Out out) {
 	case XmlTime:
 	    dump_xml_time(obj, out);
 	    break;
+	case UnixZTime:
+	    dump_time(obj, out, 1);
+	    break;
 	case UnixTime:
 	default:
-	    dump_time(obj, out);
+	    dump_time(obj, out, 0);
 	    break;
 	}
 	*out->cur++ = '}';
@@ -1271,8 +1357,17 @@ dump_obj_comp(VALUE obj, int depth, Out out) {
 	    rb_raise(rb_eTypeError, "%s.to_hash() did not return a Hash.\n", rb_class2name(rb_obj_class(obj)));
 	}
 	dump_hash(h, Qundef, depth, out->opts->mode, out);
-    } else if (Yes == out->opts->to_json && rb_respond_to(obj, oj_as_json_id)) {
-	dump_val(rb_funcall(obj, oj_as_json_id, 0), depth, out);
+    } else if (rb_respond_to(obj, oj_as_json_id)) {
+	volatile VALUE	aj = rb_funcall(obj, oj_as_json_id, 0);
+
+	// Catch the obvious brain damaged recursive dumping.
+	if (aj == obj) {
+	    volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
+
+	    dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
+	} else {
+	    dump_val(aj, depth, out);
+	}
     } else if (Yes == out->opts->to_json && rb_respond_to(obj, oj_to_json_id)) {
 	volatile VALUE	rs;
 	const char	*s;
@@ -1592,7 +1687,18 @@ dump_struct_comp(VALUE obj, int depth, Out out) {
 	    rb_raise(rb_eTypeError, "%s.to_hash() did not return a Hash.\n", rb_class2name(rb_obj_class(obj)));
 	}
 	dump_hash(h, Qundef, depth, out->opts->mode, out);
-    } else if (rb_respond_to(obj, oj_to_json_id)) {
+    } else if (rb_respond_to(obj, oj_as_json_id)) {
+	volatile VALUE	aj = rb_funcall(obj, oj_as_json_id, 0);
+
+	// Catch the obvious brain damaged recursive dumping.
+	if (aj == obj) {
+	    volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
+
+	    dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
+	} else {
+	    dump_val(aj, depth, out);
+	}
+    } else if (Yes == out->opts->to_json && rb_respond_to(obj, oj_to_json_id)) {
 	volatile VALUE	rs = rb_funcall(obj, oj_to_json_id, 0);
 	const char	*s;
 	int		len;
@@ -1605,7 +1711,9 @@ dump_struct_comp(VALUE obj, int depth, Out out) {
 	memcpy(out->cur, s, len);
 	out->cur += len;
     } else {
-	dump_struct_obj(obj, depth, out);
+	volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
+
+	dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
     }
 }
 
@@ -1630,11 +1738,35 @@ dump_struct_obj(VALUE obj, int depth, Out out) {
     *out->cur++ = '"';
     *out->cur++ = ':';
     *out->cur++ = '[';
-    fill_indent(out, d3);
-    *out->cur++ = '"';
-    memcpy(out->cur, class_name, len);
-    out->cur += len;
-    *out->cur++ = '"';
+    if ('#' == *class_name) {
+	VALUE		ma = rb_struct_s_members(clas);
+	const char	*name;
+	int		cnt = (int)RARRAY_LEN(ma);
+
+	*out->cur++ = '[';
+	for (i = 0; i < cnt; i++) {
+	    name = rb_id2name(SYM2ID(rb_ary_entry(ma, i)));
+	    len = strlen(name);
+	    size = len + 3;
+	    if (out->end - out->cur <= (long)size) {
+		grow(out, size);
+	    }
+	    if (0 < i) {
+		*out->cur++ = ',';
+	    }
+	    *out->cur++ = '"';
+	    memcpy(out->cur, name, len);
+	    out->cur += len;
+	    *out->cur++ = '"';
+	}
+	*out->cur++ = ']';
+    } else {
+	fill_indent(out, d3);
+	*out->cur++ = '"';
+	memcpy(out->cur, class_name, len);
+	out->cur += len;
+	*out->cur++ = '"';
+    }
     *out->cur++ = ',';
     size = d3 * out->indent + 2;
 #ifdef RSTRUCT_LEN
@@ -1675,6 +1807,7 @@ dump_struct_obj(VALUE obj, int depth, Out out) {
 static void
 dump_odd(VALUE obj, Odd odd, VALUE clas, int depth, Out out) {
     ID			*idp;
+    AttrGetFunc		*fp;
     volatile VALUE	v;
     const char		*name;
     size_t		size;
@@ -1702,14 +1835,46 @@ dump_odd(VALUE obj, Odd odd, VALUE clas, int depth, Out out) {
 	*out->cur++ = ',';
     }
     size = d2 * out->indent + 1;
-    for (idp = odd->attrs; 0 != *idp; idp++) {
+    for (idp = odd->attrs, fp = odd->attrFuncs; 0 != *idp; idp++, fp++) {
+	size_t	nlen;
+
 	if (out->end - out->cur <= (long)size) {
 	    grow(out, size);
 	}
 	name = rb_id2name(*idp);
-	v = rb_funcall(obj, *idp, 0);
+	nlen = strlen(name);
+	if (0 != *fp) {
+	    v = (*fp)(obj);
+	} else if (0 == strchr(name, '.')) {
+	    v = rb_funcall(obj, *idp, 0);
+	} else {
+	    char	nbuf[256];
+	    char	*n2 = nbuf;
+	    char	*n;
+	    char	*end;
+	    ID		i;
+	    
+	    if (sizeof(nbuf) <= nlen) {
+		n2 = strdup(name);
+	    } else {
+		strcpy(n2, name);
+	    }
+	    n = n2;
+	    v = obj;
+	    while (0 != (end = strchr(n, '.'))) {
+		*end = '\0';
+		i = rb_intern(n);
+		v = rb_funcall(v, i, 0);
+		n = end + 1;
+	    }
+	    i = rb_intern(n);
+	    v = rb_funcall(v, i, 0);
+	    if (nbuf != n2) {
+		free(n2);
+	    }
+	}
 	fill_indent(out, d2);
-	dump_cstr(name, strlen(name), 0, 0, out);
+	dump_cstr(name, nlen, 0, 0, out);
 	*out->cur++ = ':';
 	dump_val(v, d2, out);
 	if (out->end - out->cur <= 2) {
@@ -1734,12 +1899,13 @@ dump_val(VALUE obj, int depth, Out out) {
     if (MAX_DEPTH < depth) {
 	rb_raise(rb_eNoMemError, "Too deeply nested.\n");
     }
-    switch (rb_type(obj)) {
+    switch (type) {
     case T_NIL:		dump_nil(out);				break;
     case T_TRUE:	dump_true(out);				break;
     case T_FALSE:	dump_false(out);			break;
     case T_FIXNUM:	dump_fixnum(obj, out);			break;
     case T_FLOAT:	dump_float(obj, out);			break;
+    case T_MODULE:
     case T_CLASS:
 	switch (out->opts->mode) {
 	case StrictMode:	raise_strict(obj);		break;
@@ -1856,11 +2022,11 @@ oj_dump_obj_to_json(VALUE obj, Options copts, Out out) {
 	case '}':
 	    grow(out, 1);
 	    *out->cur++ = '\n';
-	    *out->cur = '\0';
 	default:
 	    break;
 	}
     }
+    *out->cur = '\0';
     if (Yes == copts->circular) {
 	oj_cache8_delete(out->circ_cache);
     }
@@ -2083,7 +2249,7 @@ dump_leaf_hash(Leaf leaf, int depth, Out out) {
 
 static void
 dump_leaf(Leaf leaf, int depth, Out out) {
-    switch (leaf->type) {
+    switch (leaf->rtype) {
     case T_NIL:
 	dump_nil(out);
 	break;
@@ -2109,7 +2275,7 @@ dump_leaf(Leaf leaf, int depth, Out out) {
 	dump_leaf_hash(leaf, depth, out);
 	break;
     default:
-	rb_raise(rb_eTypeError, "Unexpected type %02x.\n", leaf->type);
+	rb_raise(rb_eTypeError, "Unexpected type %02x.\n", leaf->rtype);
 	break;
     }
 }
