@@ -32,6 +32,8 @@ dflags = {
   'IS_WINDOWS' => is_windows ? 1 : 0,
   'USE_PTHREAD_MUTEX' => is_windows ? 0 : 1,
   'USE_RB_MUTEX' => (is_windows && !('1' == version[0] && '8' == version[1])) ? 1 : 0,
+  'DATETIME_1_8' => ('ruby' == type && ('1' == version[0] && '8' == version[1])) ? 1 : 0,
+  'NO_TIME_ROUND_PAD' => ('rubinius' == type) ? 1 : 0,
 }
 # This is a monster hack to get around issues with 1.9.3-p0 on CentOS 5.4. SO
 # some reason math.h and string.h contents are not processed. Might be a
