@@ -29,6 +29,7 @@
  */
 
 #include <stdio.h>
+#include <time.h>
 
 #include "oj.h"
 #include "err.h"
@@ -93,6 +94,146 @@ str_to_value(ParseInfo pi, const char *str, size_t len, const char *orig) {
     return rstr;
 }
 
+#if (RUBY_VERSION_MAJOR == 1 && RUBY_VERSION_MINOR == 8)
+static VALUE
+parse_xml_time(const char *str, int len) {
+    return rb_funcall(rb_cTime, oj_parse_id, 1, rb_str_new(str, len));
+}
+#else
+// The much faster approach (4x faster)
+static int
+parse_num(const char *str, const char *end, int cnt) {
+    int		n = 0;
+    char	c;
+    int		i;
+
+    for (i = cnt; 0 < i; i--, str++) {
+	c = *str;
+	if (end <= str || c < '0' || '9' < c) {
+	    return -1;
+	}
+	n = n * 10 + (c - '0');
+    }
+    return n;
+}
+
+static VALUE
+parse_xml_time(const char *str, int len) {
+    VALUE	args[8];
+    const char	*end = str + len;
+    int		n;
+
+    // year
+    if (0 > (n = parse_num(str, end, 4))) {
+	return Qnil;
+    }
+    str += 4;
+    args[0] = LONG2NUM(n);
+    if ('-' != *str++) {
+	return Qnil;
+    }
+    // month
+    if (0 > (n = parse_num(str, end, 2))) {
+	return Qnil;
+    }
+    str += 2;
+    args[1] = LONG2NUM(n);
+    if ('-' != *str++) {
+	return Qnil;
+    }
+    // day
+    if (0 > (n = parse_num(str, end, 2))) {
+	return Qnil;
+    }
+    str += 2;
+    args[2] = LONG2NUM(n);
+    if ('T' != *str++) {
+	return Qnil;
+    }
+    // hour
+    if (0 > (n = parse_num(str, end, 2))) {
+	return Qnil;
+    }
+    str += 2;
+    args[3] = LONG2NUM(n);
+    if (':' != *str++) {
+	return Qnil;
+    }
+    // minute
+    if (0 > (n = parse_num(str, end, 2))) {
+	return Qnil;
+    }
+    str += 2;
+    args[4] = LONG2NUM(n);
+    if (':' != *str++) {
+	return Qnil;
+    }
+    // second
+    if (0 > (n = parse_num(str, end, 2))) {
+	return Qnil;
+    }
+    str += 2;
+    if (str == end) {
+	args[5] = LONG2NUM(n);
+	args[6] = LONG2NUM(0);
+    } else {
+	char	c = *str++;
+
+	if ('.' == c) {
+	    long long	nsec = 0;
+
+	    for (; str < end; str++) {
+		c = *str;
+		if (c < '0' || '9' < c) {
+		    str++;
+		    break;
+		}
+		nsec = nsec * 10 + (c - '0');
+	    }
+	    args[5] = rb_float_new((double)n + ((double)nsec + 0.5) / 1000000000.0);
+	} else {
+	    args[5] = rb_ll2inum(n);
+	}
+	if (end < str) {
+	    args[6] = LONG2NUM(0);
+	} else {
+	    if ('Z' == c) {
+		return rb_funcall2(rb_cTime, oj_utc_id, 6, args);
+	    } else if ('+' == c) {
+		int	hr = parse_num(str, end, 2);
+		int	min;
+
+		str += 2;
+		if (0 > hr || ':' != *str++) {
+		    return Qnil;
+		}
+		min = parse_num(str, end, 2);
+		if (0 > min) {
+		    return Qnil;
+		}
+		args[6] = LONG2NUM(hr * 3600 + min * 60);
+	    } else if ('-' == c) {
+		int	hr = parse_num(str, end, 2);
+		int	min;
+
+		str += 2;
+		if (0 > hr || ':' != *str++) {
+		    return Qnil;
+		}
+		min = parse_num(str, end, 2);
+		if (0 > min) {
+		    return Qnil;
+		}
+		args[6] = LONG2NUM(-(hr * 3600 + min * 60));
+	    } else {
+		args[6] = LONG2NUM(0);
+	    }
+	}
+    }
+    return rb_funcall2(rb_cTime, oj_new_id, 7, args);
+}
+#endif
+
 static int
 hat_cstr(ParseInfo pi, Val parent, Val kval, const char *str, size_t len) {
     const char	*key = kval->key;
@@ -130,10 +271,18 @@ hat_cstr(ParseInfo pi, Val parent, Val kval, const char *str, size_t len) {
 	    parent->val = oj_encode(parent->val);
 	    break;
 	case 'c': // class
-	    parent->val = oj_name2class(pi, str, len, Yes == pi->options.auto_define);
+	    {
+		VALUE	clas = oj_name2class(pi, str, len, Yes == pi->options.auto_define);
+
+		if (Qundef == clas) {
+		    return 0;
+		} else {
+		    parent->val = clas;
+		}
+	    }
 	    break;
 	case 't': // time
-	    parent->val = rb_funcall(oj_time_class, rb_intern("parse"), 1, rb_str_new(str, len));
+	    parent->val = parse_xml_time(str, len);
 	    break;
 	default:
 	    return 0;
@@ -159,14 +308,59 @@ hat_num(ParseInfo pi, Val parent, Val kval, NumInfo ni) {
 			nsec = 1000000000LL - nsec;
 		    }
 		}
+		if (86400 == ni->exp) { // UTC time
 #if HAS_NANO_TIME
-		parent->val = rb_time_nano_new(ni->i, (long)nsec);
+		    parent->val = rb_time_nano_new(ni->i, (long)nsec);
 #else
-		parent->val = rb_time_new(ni->i, (long)(nsec / 1000));
+		    parent->val = rb_time_new(ni->i, (long)(nsec / 1000));
 #endif
-		// TBD if there is an exponent then if not gmt_offset
-		// add gmt_offset and then getgmt()
-		// if not 0 then getlocal(offset)
+		    // Since the ruby C routines alway create local time, the
+		    // offset and then a convertion to UTC keeps makes the time
+		    // match the expected value.
+		    parent->val = rb_funcall2(parent->val, oj_utc_id, 0, 0);
+		} else if (ni->hasExp) {
+		    time_t	t = (time_t)(ni->i + ni->exp);
+		    struct tm	*st = gmtime(&t);
+#if RUBY_VERSION_MAJOR == 1 && RUBY_VERSION_MINOR == 8
+		    // The only methods that allow the UTC offset to be set in
+		    // 1.8.7 is the parse() and xmlschema() methods. The
+		    // xmlschema() method always returns a Time instance that is
+		    // UTC time. (true on some platforms anyway) Time.parse()
+		    // fails on other Ruby versions until 2.2.0.
+		    char	buf[64];
+		    int		z = (0 > ni->exp ? -ni->exp : ni->exp) / 60;
+		    int		tzhour = z / 60;
+		    int		tzmin = z - tzhour * 60;
+		    int		cnt;
+
+		    cnt = sprintf(buf, "%04d-%02d-%02dT%02d:%02d:%02d.%09ld%c%02d:%02d",
+				  1900 + st->tm_year, 1 + st->tm_mon, st->tm_mday,
+				  st->tm_hour, st->tm_min, st->tm_sec, (long)nsec,
+				  (0 > ni->exp ? '-' : '+'), tzhour, tzmin);
+		    parent->val = rb_funcall(rb_cTime, oj_parse_id, 1, rb_str_new(buf, cnt));
+#else
+		    VALUE	args[8];
+
+		    args[0] = LONG2NUM(1900 + st->tm_year);
+		    args[1] = LONG2NUM(1 + st->tm_mon);
+		    args[2] = LONG2NUM(st->tm_mday);
+		    args[3] = LONG2NUM(st->tm_hour);
+		    args[4] = LONG2NUM(st->tm_min);
+#if NO_TIME_ROUND_PAD
+		    args[5] = rb_float_new((double)st->tm_sec + ((double)nsec) / 1000000000.0);
+#else
+		    args[5] = rb_float_new((double)st->tm_sec + ((double)nsec + 0.5) / 1000000000.0);
+#endif
+		    args[6] = LONG2NUM(ni->exp);
+		    parent->val = rb_funcall2(rb_cTime, oj_new_id, 7, args);
+#endif
+		} else {
+#if HAS_NANO_TIME
+		    parent->val = rb_time_nano_new(ni->i, (long)nsec);
+#else
+		    parent->val = rb_time_new(ni->i, (long)(nsec / 1000));
+#endif
+		}
 	    }
 	    break;
 	case 'i': // circular index
@@ -195,15 +389,28 @@ hat_value(ParseInfo pi, Val parent, const char *key, size_t klen, volatile VALUE
 
 	if (2 == klen && 'u' == key[1]) {
             VALUE	sc;
+	    VALUE	e1;
 
 	    if (0 == len) {
 		oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "Invalid struct data");
 		return 1;
 	    }
+	    e1 = *RARRAY_PTR(value);
+	    // check for anonymous Struct
+	    if (T_ARRAY == rb_type(e1)) {
+		VALUE	args[1024];
+		VALUE	rstr;
+		int	i, cnt = (int)RARRAY_LEN(e1);
 
-	    // If struct is not defined then we let this fail and raise an exception.
-	    sc = oj_name2struct(pi, *RARRAY_PTR(value));
-
+		for (i = 0; i < cnt; i++) {
+		    rstr = rb_ary_entry(e1, i);
+		    args[i] = rb_funcall(rstr, oj_to_sym_id, 0);
+		}
+		sc = rb_funcall2(rb_cStruct, oj_new_id, cnt, args);
+	    } else {
+		// If struct is not defined then we let this fail and raise an exception.
+		sc = oj_name2struct(pi, *RARRAY_PTR(value));
+	    }
             // Create a properly initialized struct instance without calling the initialize method.
             parent->val = rb_obj_alloc(sc);
             // If the JSON array has more entries than the struct class allows, we record an error.
