@@ -34,7 +34,14 @@
 
 static struct _Odd	_odds[4]; // bump up if new initial Odd classes are added
 static struct _Odd	*odds = _odds;
-static int		odd_cnt = 0;
+static long		odd_cnt = 0;
+static ID		sec_id;
+static ID		sec_fraction_id;
+static ID		to_f_id;
+static ID		numerator_id;
+static ID		denominator_id;
+static ID		rational_id;
+static VALUE		rational_class;
 
 static void
 set_class(Odd odd, const char *classname) {
@@ -52,11 +59,36 @@ set_class(Odd odd, const char *classname) {
     *idp = 0;
 }
 
+static VALUE
+get_datetime_secs(VALUE obj) {
+    VALUE	rsecs = rb_funcall(obj, sec_id, 0);
+    VALUE	rfrac = rb_funcall(obj, sec_fraction_id, 0);
+    long	sec = NUM2LONG(rsecs);
+    long long	num = rb_num2ll(rb_funcall(rfrac, numerator_id, 0));
+    long long	den = rb_num2ll(rb_funcall(rfrac, denominator_id, 0));
+
+#if DATETIME_1_8
+    num *= 86400;
+#endif
+    num += sec * den;
+
+    return rb_funcall(rb_cObject, rational_id, 2, rb_ll2inum(num), rb_ll2inum(den));
+}
+
 void
 oj_odd_init() {
     Odd		odd;
     const char	**np;
 
+    sec_id = rb_intern("sec");
+    sec_fraction_id = rb_intern("sec_fraction");
+    to_f_id = rb_intern("to_f");
+    numerator_id = rb_intern("numerator");
+    denominator_id = rb_intern("denominator");
+    rational_id = rb_intern("Rational");
+    rational_class = rb_const_get(rb_cObject, rational_id);
+
+    memset(_odds, 0, sizeof(_odds));
     odd = odds;
     // Rational
     np = odd->attr_names;
@@ -65,7 +97,7 @@ oj_odd_init() {
     *np = 0;
     set_class(odd, "Rational");
     odd->create_obj = rb_cObject;
-    odd->create_op = rb_intern("Rational");
+    odd->create_op = rational_id;
     odd->attr_cnt = 2;
     // Date
     odd++;
@@ -91,6 +123,7 @@ oj_odd_init() {
     *np++ = 0;
     set_class(odd, "DateTime");
     odd->attr_cnt = 8;
+    odd->attrFuncs[5] = get_datetime_secs;
     // Range
     odd++;
     np = odd->attr_names;
@@ -166,6 +199,7 @@ oj_reg_odd(VALUE clas, VALUE create_object, VALUE create_method, int mcnt, VALUE
     Odd		odd;
     const char	**np;
     ID		*ap;
+    AttrGetFunc	*fp;
 
     if (_odds == odds) {
 	odds = ALLOC_N(struct _Odd, odd_cnt + 1);
@@ -181,7 +215,8 @@ oj_reg_odd(VALUE clas, VALUE create_object, VALUE create_method, int mcnt, VALUE
     odd->create_obj = create_object;
     odd->create_op = SYM2ID(create_method);
     odd->attr_cnt = mcnt;
-    for (ap = odd->attrs, np = odd->attr_names; 0 < mcnt; mcnt--, ap++, np++, members++) {
+    for (ap = odd->attrs, np = odd->attr_names, fp = odd->attrFuncs; 0 < mcnt; mcnt--, ap++, np++, members++, fp++) {
+	*fp = 0;
 	switch (rb_type(*members)) {
 	case T_STRING:
 	    *np = strdup(rb_string_value_ptr(members));
