@@ -47,8 +47,9 @@
 #else
 #define NUM_MAX		(FIXNUM_MAX >> 8)
 #endif
-#define EXP_MAX		1023
-#define DEC_MAX		14
+//#define EXP_MAX		1023
+#define EXP_MAX		100000
+#define DEC_MAX		15
 
 static void
 next_non_white(ParseInfo pi) {
@@ -70,11 +71,11 @@ static void
 skip_comment(ParseInfo pi) {
     if ('*' == *pi->cur) {
 	pi->cur++;
-	for (; '\0' != *pi->cur; pi->cur++) {
+	for (; pi->cur < pi->end; pi->cur++) {
 	    if ('*' == *pi->cur && '/' == *(pi->cur + 1)) {
 		pi->cur += 2;
 		return;
-	    } else if ('\0' == *pi->cur) {
+	    } else if (pi->end <= pi->cur) {
 		oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "comment not terminated");
 		return;
 	    }
@@ -110,8 +111,8 @@ add_value(ParseInfo pi, VALUE rval) {
 	    parent->next = NEXT_ARRAY_COMMA;
 	    break;
 	case NEXT_HASH_VALUE:
-	    pi->hash_set_value(pi, parent->key, parent->klen, rval);
-	    if (0 != parent->key && (parent->key < pi->json || pi->cur < parent->key)) {
+	    pi->hash_set_value(pi, parent, rval);
+	    if (0 != parent->key && 0 < parent->klen && (parent->key < pi->json || pi->cur < parent->key)) {
 		xfree((char*)parent->key);
 		parent->key = 0;
 	    }
@@ -226,7 +227,7 @@ read_escaped_str(ParseInfo pi, const char *start) {
 	buf_append_string(&buf, start, cnt);
     }
     for (s = pi->cur; '"' != *s; s++) {
-	if ('\0' == *s) {
+	if (s >= pi->end) {
 	    oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "quoted string not terminated");
 	    buf_cleanup(&buf);
 	    return;
@@ -295,15 +296,19 @@ read_escaped_str(ParseInfo pi, const char *start) {
 	    break;
 	case NEXT_HASH_NEW:
 	case NEXT_HASH_KEY:
-	    // key will not be between pi->json and pi->cur.
-	    parent->key = strdup(buf.head);
-	    parent->klen = buf_len(&buf);
+	    if (Qundef == (parent->key_val = pi->hash_key(pi, buf.head, buf_len(&buf)))) {
+		parent->key = strdup(buf.head);
+		parent->klen = buf_len(&buf);
+	    } else {
+		parent->key = "";
+		parent->klen = 0;
+	    }
 	    parent->k1 = *start;
 	    parent->next = NEXT_HASH_COLON;
 	    break;
 	case NEXT_HASH_VALUE:
-	    pi->hash_set_cstr(pi, parent->key, parent->klen, buf.head, buf_len(&buf), start);
-	    if (0 != parent->key && (parent->key < pi->json || pi->cur < parent->key)) {
+	    pi->hash_set_cstr(pi, parent, buf.head, buf_len(&buf), start);
+	    if (0 != parent->key && 0 < parent->klen && (parent->key < pi->json || pi->cur < parent->key)) {
 		xfree((char*)parent->key);
 		parent->key = 0;
 	    }
@@ -328,9 +333,12 @@ read_str(ParseInfo pi) {
     Val		parent = stack_peek(&pi->stack);
 
     for (; '"' != *pi->cur; pi->cur++) {
-	if ('\0' == *pi->cur) {
+	if (pi->end <= pi->cur) {
 	    oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "quoted string not terminated");
 	    return;
+	} else if ('\0' == *pi->cur) {
+	    oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "NULL byte in string");
+	    return;
 	} else if ('\\' == *pi->cur) {
 	    read_escaped_str(pi, str);
 	    return;
@@ -347,14 +355,19 @@ read_str(ParseInfo pi) {
 	    break;
 	case NEXT_HASH_NEW:
 	case NEXT_HASH_KEY:
-	    parent->key = str;
-	    parent->klen = pi->cur - str;
+	    if (Qundef == (parent->key_val = pi->hash_key(pi, str, pi->cur - str))) {
+		parent->key = str;
+		parent->klen = pi->cur - str;
+	    } else {
+		parent->key = "";
+		parent->klen = 0;
+	    }
 	    parent->k1 = *str;
 	    parent->next = NEXT_HASH_COLON;
 	    break;
 	case NEXT_HASH_VALUE:
-	    pi->hash_set_cstr(pi, parent->key, parent->klen, str, pi->cur - str, str);
-	    if (0 != parent->key && (parent->key < pi->json || pi->cur < parent->key)) {
+	    pi->hash_set_cstr(pi, parent, str, pi->cur - str, str);
+	    if (0 != parent->key && 0 < parent->klen && (parent->key < pi->json || pi->cur < parent->key)) {
 		xfree((char*)parent->key);
 		parent->key = 0;
 	    }
@@ -389,6 +402,7 @@ read_num(ParseInfo pi) {
     ni.infinity = 0;
     ni.nan = 0;
     ni.neg = 0;
+    ni.hasExp = 0;
     ni.no_big = (FloatDec == pi->options.bigdec_load);
 
     if ('-' == *pi->cur) {
@@ -424,6 +438,7 @@ read_num(ParseInfo pi) {
 		} else {
 		    zero_cnt = 0;
 		}
+		// TBD move size check here
 		ni.i = ni.i * 10 + d;
 		if (LONG_MAX <= ni.i || DEC_MAX < ni.dec_cnt - zero_cnt) {
 		    ni.big = 1;
@@ -441,6 +456,7 @@ read_num(ParseInfo pi) {
 		    zero_cnt = 0;
 		}
 		ni.dec_cnt++;
+		// TBD move size check here
 		ni.num = ni.num * 10 + d;
 		ni.div *= 10;
 		if (LONG_MAX <= ni.div || DEC_MAX < ni.dec_cnt - zero_cnt) {
@@ -451,6 +467,7 @@ read_num(ParseInfo pi) {
 	if ('e' == *pi->cur || 'E' == *pi->cur) {
 	    int	eneg = 0;
 
+	    ni.hasExp = 1;
 	    pi->cur++;
 	    if ('-' == *pi->cur) {
 		pi->cur++;
@@ -484,8 +501,8 @@ read_num(ParseInfo pi) {
 	    parent->next = NEXT_ARRAY_COMMA;
 	    break;
 	case NEXT_HASH_VALUE:
-	    pi->hash_set_num(pi, parent->key, parent->klen, &ni);
-	    if (0 != parent->key && (parent->key < pi->json || pi->cur < parent->key)) {
+	    pi->hash_set_num(pi, parent, &ni);
+	    if (0 != parent->key && 0 < parent->klen && (parent->key < pi->json || pi->cur < parent->key)) {
 		xfree((char*)parent->key);
 		parent->key = 0;
 	    }
@@ -570,10 +587,15 @@ colon(ParseInfo pi) {
 
 void
 oj_parse2(ParseInfo pi) {
+    int	first = 1;
+
     pi->cur = pi->json;
     err_init(&pi->err);
     while (1) {
 	next_non_white(pi);
+	if (!first && '\0' != *pi->cur) {
+	    rb_raise(rb_eSyntaxError, "Extra characters after the JSON document.");
+	}
 	switch (*pi->cur++) {
 	case '{':
 	    hash_start(pi);
@@ -640,18 +662,23 @@ oj_parse2(ParseInfo pi) {
 	if (err_has(&pi->err)) {
 	    return;
 	}
-	if (Qundef != pi->proc && stack_empty(&pi->stack)) {
-	    if (Qnil == pi->proc) {
-		rb_yield(stack_head_val(&pi->stack));
-	    } else {
+	if (stack_empty(&pi->stack)) {
+	    if (Qundef != pi->proc) {
+		if (Qnil == pi->proc) {
+		    rb_yield(stack_head_val(&pi->stack));
+		} else {
 #if HAS_PROC_WITH_BLOCK
-		VALUE	args[1];
+		    VALUE	args[1];
 
-		*args = stack_head_val(&pi->stack);
-		rb_proc_call_with_block(pi->proc, 1, args, Qnil);
+		    *args = stack_head_val(&pi->stack);
+		    rb_proc_call_with_block(pi->proc, 1, args, Qnil);
 #else
-		rb_raise(rb_eNotImpError, "Calling a Proc with a block not supported in this version. Use func() {|x| } syntax instead.");
+		    rb_raise(rb_eNotImpError,
+			     "Calling a Proc with a block not supported in this version. Use func() {|x| } syntax instead.");
 #endif
+		}
+	    } else {
+		first = 0;
 	    }
 	}
     }
@@ -687,9 +714,9 @@ oj_num_as_value(NumInfo ni) {
 	    }
 	} else {
 	    if (ni->neg) {
-		rnum = LONG2NUM(-ni->i);
+		rnum = rb_ll2inum(-ni->i);
 	    } else {
-		rnum = LONG2NUM(ni->i);
+		rnum = rb_ll2inum(ni->i);
 	    }
 	}
     } else { // decimal
@@ -699,13 +726,13 @@ oj_num_as_value(NumInfo ni) {
 		rnum = rb_funcall(rnum, rb_intern("to_f"), 0);
 	    }
 	} else {
-	    double	d = (double)ni->i + (double)ni->num / (double)ni->div;
+	    double	d = (double)ni->i + (double)ni->num * (1.0L / ni->div);
 
 	    if (ni->neg) {
 		d = -d;
 	    }
 	    if (0 != ni->exp) {
-		d *= pow(10.0, ni->exp);
+		d *= pow(10.0L, ni->exp);
 	    }
 	    rnum = rb_float_new(d);
 	}
@@ -736,6 +763,12 @@ protect_parse(VALUE pip) {
     return Qnil;
 }
 
+void
+oj_pi_set_input_str(ParseInfo pi, volatile VALUE input) {
+    pi->json = rb_string_value_ptr((VALUE*)&input);
+    pi->end = pi->json + RSTRING_LEN(input);
+}
+
 VALUE
 oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len, int yieldOk) {
     char		*buf = 0;
@@ -757,31 +790,24 @@ oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len, int yie
     } else {
 	pi->proc = Qundef;
     }
-    pi->cbc = (void*)0;
     if (0 != json) {
 	pi->json = json;
 	pi->end = json + len;
 	free_json = 1;
     } else if (T_STRING == rb_type(input)) {
-	pi->json = rb_string_value_cstr((VALUE*)&input);
-	pi->end = pi->json + RSTRING_LEN(input);
+	oj_pi_set_input_str(pi, input);
     } else if (Qnil == input && Yes == pi->options.nilnil) {
 	return Qnil;
     } else {
 	VALUE		clas = rb_obj_class(input);
 	volatile VALUE	s;
-#if !IS_WINDOWS
-	int		fd;
-#endif
 
 	if (oj_stringio_class == clas) {
 	    s = rb_funcall2(input, oj_string_id, 0, 0);
-	    pi->json = rb_string_value_cstr((VALUE*)&s);
-	    pi->end = pi->json + RSTRING_LEN(s);
+	    oj_pi_set_input_str(pi, s);
 #if !IS_WINDOWS
-	} else if (rb_respond_to(input, oj_fileno_id) &&
-		   Qnil != (s = rb_funcall(input, oj_fileno_id, 0)) &&
-		   0 != (fd = FIX2INT(s))) {
+	} else if (rb_cFile == clas && 0 == FIX2INT(rb_funcall(input, oj_pos_id, 0))) {
+	    int		fd = FIX2INT(rb_funcall(input, oj_fileno_id, 0));
 	    ssize_t	cnt;
 	    size_t	len = lseek(fd, 0, SEEK_END);
 
@@ -806,7 +832,7 @@ oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len, int yie
 	    return oj_pi_sparse(argc, argv, pi, 0);
 	} else {
 	    rb_raise(rb_eArgError, "strict_parse() expected a String or IO Object.");
-	}
+}
     }
     if (Yes == pi->options.circular) {
 	pi->circ_array = oj_circ_array_new();
@@ -866,5 +892,21 @@ oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len, int yie
     if (err_has(&pi->err)) {
 	oj_err_raise(&pi->err);
     }
+    if (pi->options.quirks_mode == No) {
+	switch (rb_type(result)) {
+            case T_NIL:
+	    case T_TRUE:
+	    case T_FALSE:
+	    case T_FIXNUM:
+	    case T_FLOAT:
+	    case T_CLASS:
+	    case T_SYMBOL:
+		rb_raise(oj_parse_error_class, "unexpected non-document value");
+		break;
+	    default:
+		// okay
+		break;
+	}
+    }
     return result;
 }
