@@ -69,9 +69,9 @@ static void	next_non_white(ParseInfo pi);
 static char*	read_quoted_value(ParseInfo pi);
 static void	skip_comment(ParseInfo pi);
 
-/* This XML parser is a single pass, destructive, callback parser. It is a
+/* This JSON parser is a single pass, destructive, callback parser. It is a
  * single pass parse since it only make one pass over the characters in the
- * XML document string. It is destructive because it re-uses the content of
+ * JSON document string. It is destructive because it re-uses the content of
  * the string for values in the callback and places \0 characters at various
  * places to mark the end of tokens and strings. It is a callback parser like
  * a SAX parser because it uses callback when document elements are
@@ -627,22 +627,6 @@ read_quoted_value(ParseInfo pi) {
     return value;
 }
 
-inline static int
-respond_to(VALUE obj, ID method) {
-#ifdef JRUBY_RUBY
-    /* There is a bug in JRuby where rb_respond_to() returns true (1) even if
-     * a method is private. */
-    {
-	VALUE	args[1];
-
-	*args = ID2SYM(method);
-	return (Qtrue == rb_funcall2(obj, rb_intern("respond_to?"), 1, args));
-    }
-#else
-    return rb_respond_to(obj, method);
-#endif
-}
-
 static void
 saj_parse(VALUE handler, char *json) {
     volatile VALUE	obj = Qnil;
@@ -675,12 +659,12 @@ saj_parse(VALUE handler, char *json) {
     }
 #endif
     pi.handler = handler;
-    pi.has_hash_start = respond_to(handler, oj_hash_start_id);
-    pi.has_hash_end = respond_to(handler, oj_hash_end_id);
-    pi.has_array_start = respond_to(handler, oj_array_start_id);
-    pi.has_array_end = respond_to(handler, oj_array_end_id);
-    pi.has_add_value = respond_to(handler, oj_add_value_id);
-    pi.has_error = respond_to(handler, oj_error_id);
+    pi.has_hash_start = rb_respond_to(handler, oj_hash_start_id);
+    pi.has_hash_end = rb_respond_to(handler, oj_hash_end_id);
+    pi.has_array_start = rb_respond_to(handler, oj_array_start_id);
+    pi.has_array_end = rb_respond_to(handler, oj_array_end_id);
+    pi.has_add_value = rb_respond_to(handler, oj_add_value_id);
+    pi.has_error = rb_respond_to(handler, oj_error_id);
     read_next(&pi, 0);
     next_non_white(&pi);
     if ('\0' != *pi.s) {
@@ -716,9 +700,6 @@ oj_saj_parse(int argc, VALUE *argv, VALUE self) {
     } else {
 	VALUE		clas = rb_obj_class(input);
 	volatile VALUE	s;
-#if !IS_WINDOWS
-	int		fd;
-#endif
 	
 	if (oj_stringio_class == clas) {
 	    s = rb_funcall2(input, oj_string_id, 0, 0);
@@ -726,9 +707,8 @@ oj_saj_parse(int argc, VALUE *argv, VALUE self) {
 	    json = ALLOC_N(char, len);
 	    strcpy(json, rb_string_value_cstr((VALUE*)&s));
 #if !IS_WINDOWS
-	} else if (rb_respond_to(input, oj_fileno_id) &&
-		   Qnil != (s = rb_funcall(input, oj_fileno_id, 0)) &&
-		   0 != (fd = FIX2INT(s))) {
+	} else if (rb_cFile == clas && 0 == FIX2INT(rb_funcall(input, oj_pos_id, 0))) {
+	    int		fd = FIX2INT(rb_funcall(input, oj_fileno_id, 0));
 	    ssize_t	cnt;
 
 	    len = lseek(fd, 0, SEEK_END);
