@@ -48,7 +48,7 @@
 #else
 #define NUM_MAX		(FIXNUM_MAX >> 8)
 #endif
-#define EXP_MAX		1023
+#define EXP_MAX		100000
 #define DEC_MAX		15
 
 static void
@@ -413,6 +413,7 @@ read_num(ParseInfo pi) {
     ni.infinity = 0;
     ni.nan = 0;
     ni.neg = 0;
+    ni.hasExp = 0;
     ni.no_big = (FloatDec == pi->options.bigdec_load);
     c = reader_get(&pi->rd);
     if ('-' == c) {
@@ -467,6 +468,7 @@ read_num(ParseInfo pi) {
 	if ('e' == c || 'E' == c) {
 	    int	eneg = 0;
 
+	    ni.hasExp = 1;
 	    c = reader_get(&pi->rd);
 	    if ('-' == c) {
 		c = reader_get(&pi->rd);
@@ -486,7 +488,9 @@ read_num(ParseInfo pi) {
 	}
 	ni.dec_cnt -= zero_cnt;
 	ni.len = pi->rd.tail - pi->rd.str;
-	reader_backup(&pi->rd);
+	if (0 != c) {
+	    reader_backup(&pi->rd);
+	}
     }
     if (BigDec == pi->options.bigdec_load) {
 	ni.big = 1;
@@ -598,11 +602,15 @@ colon(ParseInfo pi) {
 
 void
 oj_sparse2(ParseInfo pi) {
+    int		first = 1;
     char	c;
 
     err_init(&pi->err);
     while (1) {
 	c = reader_next_non_white(&pi->rd);
+	if (!first && '\0' != c) {
+	    rb_raise(rb_eSyntaxError, "Extra characters after the JSON document.");
+	}
 	switch (c) {
 	case '{':
 	    hash_start(pi);
@@ -697,20 +705,24 @@ oj_sparse2(ParseInfo pi) {
 	if (err_has(&pi->err)) {
 	    return;
 	}
-	if (Qundef != pi->proc && stack_empty(&pi->stack)) {
-	    if (Qnil == pi->proc) {
-		rb_yield(stack_head_val(&pi->stack));
-	    } else {
+	if (stack_empty(&pi->stack)) {
+	    if (Qundef != pi->proc) {
+		if (Qnil == pi->proc) {
+		    rb_yield(stack_head_val(&pi->stack));
+		} else {
 #if HAS_PROC_WITH_BLOCK
-		VALUE	args[1];
+		    VALUE	args[1];
 
-		*args = stack_head_val(&pi->stack);
-		rb_proc_call_with_block(pi->proc, 1, args, Qnil);
+		    *args = stack_head_val(&pi->stack);
+		    rb_proc_call_with_block(pi->proc, 1, args, Qnil);
 #else
-		oj_set_error_at(pi, rb_eNotImpError, __FILE__, __LINE__,
-				 "Calling a Proc with a block not supported in this version. Use func() {|x| } syntax instead.");
-		return;
+		    oj_set_error_at(pi, rb_eNotImpError, __FILE__, __LINE__,
+				    "Calling a Proc with a block not supported in this version. Use func() {|x| } syntax instead.");
+		    return;
 #endif
+		}
+	    } else {
+		first = 0;
 	    }
 	}
     }
