@@ -38,13 +38,15 @@
 #include "parse.h"
 #include "encode.h"
 
-// Workaround in case INFINITY is not defined in math.h or if the OS is CentOS
-#define OJ_INFINITY (1.0/0.0)
-
 static void
 noop_end(struct _ParseInfo *pi) {
 }
 
+static VALUE
+noop_hash_key(struct _ParseInfo *pi, const char *key, size_t klen) {
+    return Qundef;
+}
+
 static void
 add_value(ParseInfo pi, VALUE val) {
     pi->stack.head->val = val;
@@ -60,6 +62,9 @@ add_cstr(ParseInfo pi, const char *str, size_t len, const char *orig) {
 
 static void
 add_num(ParseInfo pi, NumInfo ni) {
+    if (ni->infinity || ni->nan) {
+	oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "not a number or other value");
+    }
     pi->stack.head->val = oj_num_as_value(ni);
 }
 
@@ -69,9 +74,12 @@ start_hash(ParseInfo pi) {
 }
 
 static VALUE
-hash_key(ParseInfo pi, const char *key, size_t klen) {
-    volatile VALUE	rkey = rb_str_new(key, klen);
+calc_hash_key(ParseInfo pi, Val parent) {
+    volatile VALUE	rkey = parent->key_val;
 
+    if (Qundef == rkey) {
+	rkey = rb_str_new(parent->key, parent->klen);
+    }
     rkey = oj_encode(rkey);
     if (Yes == pi->options.sym_key) {
 	rkey = rb_str_intern(rkey);
@@ -80,21 +88,24 @@ hash_key(ParseInfo pi, const char *key, size_t klen) {
 }
 
 static void
-hash_set_cstr(ParseInfo pi, const char *key, size_t klen, const char *str, size_t len, const char *orig) {
+hash_set_cstr(ParseInfo pi, Val parent, const char *str, size_t len, const char *orig) {
     volatile VALUE	rstr = rb_str_new(str, len);
 
     rstr = oj_encode(rstr);
-    rb_hash_aset(stack_peek(&pi->stack)->val, hash_key(pi, key, klen), rstr);
+    rb_hash_aset(stack_peek(&pi->stack)->val, calc_hash_key(pi, parent), rstr);
 }
 
 static void
-hash_set_num(struct _ParseInfo *pi, const char *key, size_t klen, NumInfo ni) {
-    rb_hash_aset(stack_peek(&pi->stack)->val, hash_key(pi, key, klen), oj_num_as_value(ni));
+hash_set_num(struct _ParseInfo *pi, Val parent, NumInfo ni) {
+    if (ni->infinity || ni->nan) {
+	oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "not a number or other value");
+    }
+    rb_hash_aset(stack_peek(&pi->stack)->val, calc_hash_key(pi, parent), oj_num_as_value(ni));
 }
 
 static void
-hash_set_value(ParseInfo pi, const char *key, size_t klen, VALUE value) {
-    rb_hash_aset(stack_peek(&pi->stack)->val, hash_key(pi, key, klen), value);
+hash_set_value(ParseInfo pi, Val parent, VALUE value) {
+    rb_hash_aset(stack_peek(&pi->stack)->val, calc_hash_key(pi, parent), value);
 }
 
 static VALUE
@@ -112,6 +123,9 @@ array_append_cstr(ParseInfo pi, const char *str, size_t len, const char *orig) {
 
 static void
 array_append_num(ParseInfo pi, NumInfo ni) {
+    if (ni->infinity || ni->nan) {
+	oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "not a number or other value");
+    }
     rb_ary_push(stack_peek(&pi->stack)->val, oj_num_as_value(ni));
 }
 
@@ -124,6 +138,7 @@ void
 oj_set_strict_callbacks(ParseInfo pi) {
     pi->start_hash = start_hash;
     pi->end_hash = noop_end;
+    pi->hash_key = noop_hash_key;
     pi->hash_set_cstr = hash_set_cstr;
     pi->hash_set_num = hash_set_num;
     pi->hash_set_value = hash_set_value;
@@ -143,6 +158,7 @@ oj_strict_parse(int argc, VALUE *argv, VALUE self) {
     struct _ParseInfo	pi;
 
     pi.options = oj_default_options;
+    pi.handler = Qnil;
     oj_set_strict_callbacks(&pi);
 
     if (T_STRING == rb_type(*argv)) {
@@ -157,6 +173,7 @@ oj_strict_parse_cstr(int argc, VALUE *argv, char *json, size_t len) {
     struct _ParseInfo	pi;
 
     pi.options = oj_default_options;
+    pi.handler = Qnil;
     oj_set_strict_callbacks(&pi);
 
     return oj_pi_parse(argc, argv, &pi, json, len, 1);
