@@ -24,6 +24,14 @@
 module Oj
 end
 
+begin
+  # This require exists to get around Rubinius failing to load bigdecimal from
+  # the C extension.
+  require 'bigdecimal'
+rescue Exception
+  # ignore
+end
+
 require 'oj/version'
 require 'oj/bag'
 require 'oj/error'
