@@ -7,8 +7,9 @@ require 'helper'
 
 begin
   require 'rails/all'
-rescue Exception
-  Process.exit!(true)
+rescue Exception => e
+  puts "*** #{e.class}: #{e.message}"
+  Process.exit!(false)
 end
 
 Oj.mimic_JSON
