@@ -5,7 +5,7 @@ $: << File.dirname(__FILE__)
 
 require 'helper'
 
-$json1 = %{{
+$json1 = %|{
   "array": [
     {
       "num"   : 3,
@@ -18,7 +18,7 @@ $json1 = %{{
     }
   ],
   "boolean" : true
-}}
+}|
 
 class DocTest < Minitest::Test
   def setup
@@ -61,6 +61,22 @@ class DocTest < Minitest::Test
     end
   end
 
+  def test_encoding
+    json = %{"ぴーたー"}
+    Oj::Doc.open(json) do |doc|
+      assert_equal(String, doc.type)
+      assert_equal("ぴーたー", doc.fetch())
+    end
+  end
+
+  def test_encoding_escaped
+    json = %{"\\u3074\\u30fc\\u305f\\u30fc"}
+    Oj::Doc.open(json) do |doc|
+      assert_equal(String, doc.type)
+      assert_equal("ぴーたー", doc.fetch())
+    end
+  end
+
   def test_fixnum
     json = %{12345}
     Oj::Doc.open(json) do |doc|
@@ -243,6 +259,18 @@ class DocTest < Minitest::Test
     end
   end
 
+  def test_move_fetch_path
+    Oj::Doc.open($json1) do |doc|
+      [['/array/1', 'num', 3],
+       ['/array/1', 'string', 'message'],
+       ['/array/1/hash', 'h2/a', [1, 2, 3]],
+      ].each do |path,fetch_path,val|
+        doc.move(path)
+        assert_equal(val, doc.fetch(fetch_path))
+      end
+    end
+  end
+
   def test_home
     Oj::Doc.open($json1) do |doc|
       doc.move('/array/1/num')
