@@ -7,6 +7,9 @@ require 'helper'
 
 class Juice < Minitest::Test
 
+  module TestModule
+  end
+
   class Jam
     attr_accessor :x, :y
 
@@ -87,57 +90,67 @@ class Juice < Minitest::Test
     Oj.default_options = @default_options
   end
 
-  def test0_get_options
+=begin
+  # Depending on the order the values may have changed. The set_options sets
+  # should cover the function itself.
+  def test_get_options
     opts = Oj.default_options()
     assert_equal({ :indent=>0,
                    :second_precision=>9,
                    :circular=>false,
+                   :class_cache=>true,
                    :auto_define=>false,
                    :symbol_keys=>false,
-                   :class_cache=>true,
-                   :escape_mode=>:json,
-                   :mode=>:object,
-                   :time_format=>:unix,
                    :bigdecimal_as_decimal=>true,
-                   :bigdecimal_load=>:auto,
-                   :use_to_json=>false,
+                   :use_to_json=>true,
                    :nilnil=>false,
                    :allow_gc=>true,
+                   :quirks_mode=>true,
+                   :float_precision=>15,
+                   :mode=>:object,
+                   :escape_mode=>:json,
+                   :time_format=>:unix,
+                   :bigdecimal_load=>:auto,
                    :create_id=>'json_class'}, opts)
   end
+=end
 
-  def test0_set_options
-    orig = {
+  def test_set_options
+    orig ={
       :indent=>0,
       :second_precision=>9,
       :circular=>false,
+      :class_cache=>true,
       :auto_define=>false,
       :symbol_keys=>false,
-      :class_cache=>true,
-      :escape_mode=>:ascii,
-      :mode=>:object,
-      :time_format=>:unix,
       :bigdecimal_as_decimal=>true,
-      :bigdecimal_load=>:auto,
       :use_to_json=>true,
       :nilnil=>false,
       :allow_gc=>true,
+      :quirks_mode=>true,
+      :float_precision=>15,
+      :mode=>:object,
+      :escape_mode=>:json,
+      :time_format=>:unix,
+      :bigdecimal_load=>:auto,
       :create_id=>'json_class'}
     o2 = {
       :indent=>4,
       :second_precision=>7,
       :circular=>true,
+      :class_cache=>false,
       :auto_define=>true,
       :symbol_keys=>true,
-      :class_cache=>false,
-      :escape_mode=>:json,
-      :mode=>:compat,
-      :time_format=>:ruby,
       :bigdecimal_as_decimal=>false,
-      :bigdecimal_load=>:bigdecimal,
       :use_to_json=>false,
       :nilnil=>true,
       :allow_gc=>false,
+      :quirks_mode=>true,
+      :float_precision=>15,
+      :mode=>:compat,
+      :escape_mode=>:json,
+      :time_format=>:ruby,
+      :bigdecimal_load=>:bigdecimal,
       :create_id=>nil}
     o3 = { :indent => 4 }
     Oj.default_options = o2
@@ -166,6 +179,33 @@ class Juice < Minitest::Test
     dump_and_load(1, false)
   end
 
+  def test_float_dump
+    Oj.default_options = { :float_precision => 16 }
+    assert_equal('1405460727.723866', Oj.dump(1405460727.723866))
+    Oj.default_options = { :float_precision => 5 }
+    assert_equal('1.4055', Oj.dump(1.405460727))
+    Oj.default_options = { :float_precision => 0 }
+    if RUBY_VERSION.start_with?('1.8')
+      assert_equal('1405460727.72387', Oj.dump(1405460727.723866))
+    else
+      assert_equal('1405460727.723866', Oj.dump(1405460727.723866))
+    end
+    Oj.default_options = { :float_precision => 15 }
+    assert_equal('0.56', Oj.dump(0.56))
+    assert_equal('0.5773', Oj.dump(0.5773))
+    assert_equal('0.6768', Oj.dump(0.6768))
+    assert_equal('0.685', Oj.dump(0.685))
+    assert_equal('0.7032', Oj.dump(0.7032))
+    assert_equal('0.7051', Oj.dump(0.7051))
+    assert_equal('0.8274', Oj.dump(0.8274))
+    assert_equal('0.9149', Oj.dump(0.9149))
+    assert_equal('64.4', Oj.dump(64.4))
+    assert_equal('71.6', Oj.dump(71.6))
+    assert_equal('73.4', Oj.dump(73.4))
+    assert_equal('80.6', Oj.dump(80.6))
+    assert_equal('-95.640172', Oj.dump(-95.640172))
+  end
+
   def test_float
     mode = Oj.default_options()[:mode]
     Oj.default_options = {:mode => :object}
@@ -204,15 +244,12 @@ class Juice < Minitest::Test
   def test_encode
     opts = Oj.default_options
     Oj.default_options = { :ascii_only => false }
-    unless 'jruby' == $ruby
-      dump_and_load("ぴーたー", false)
-    end
+    dump_and_load("ぴーたー", false)
+
     Oj.default_options = { :ascii_only => true }
     json = Oj.dump("ぴーたー")
     assert_equal(%{"\\u3074\\u30fc\\u305f\\u30fc"}, json)
-    unless 'jruby' == $ruby
-      dump_and_load("ぴーたー", false)
-    end
+    dump_and_load("ぴーたー", false)
     Oj.default_options = opts
   end
 
@@ -246,6 +283,31 @@ class Juice < Minitest::Test
     assert(false, "*** expected an exception")
   end
 
+  # multiple JSON in one string
+  def test_multiple_json_callback
+    json = %{{"a":1}
+[1,2][3,4]
+{"b":2}
+}
+    results = []
+    Oj.load(json) { |x| results << x }
+    assert_equal([{"a"=>1}, [1,2], [3,4], {"b"=>2}], results)
+  end
+
+  def test_multiple_json_no_callback
+    json = %{{"a":1}
+[1,2][3,4]
+{"b":2}
+}
+    begin
+      Oj.load(json)
+    rescue Exception
+      assert(true)
+      return
+    end
+    assert(false, "*** expected an exception")
+  end
+
   # encoding tests
   def test_does_not_escape_entities_by_default
     Oj.default_options = { :escape_mode => :ascii } # set in mimic mode
@@ -335,26 +397,26 @@ class Juice < Minitest::Test
   def test_unix_time_compat
     t = Time.xmlschema("2012-01-05T23:58:07.123456000+09:00")
     #t = Time.local(2012, 1, 5, 23, 58, 7, 123456)
-    json = Oj.dump(t, :mode => :compat)
+    json = Oj.dump(t, :mode => :compat, :time_format => :unix)
     assert_equal(%{1325775487.123456000}, json)
   end
   def test_unix_time_compat_precision
     t = Time.xmlschema("2012-01-05T23:58:07.123456789+09:00")
     #t = Time.local(2012, 1, 5, 23, 58, 7, 123456)
-    json = Oj.dump(t, :mode => :compat, :second_precision => 5)
+    json = Oj.dump(t, :mode => :compat, :second_precision => 5, :time_format => :unix)
     assert_equal(%{1325775487.12346}, json)
     t = Time.xmlschema("2012-01-05T23:58:07.999600+09:00")
-    json = Oj.dump(t, :mode => :compat, :second_precision => 3)
+    json = Oj.dump(t, :mode => :compat, :second_precision => 3, :time_format => :unix)
     assert_equal(%{1325775488.000}, json)
   end
   def test_unix_time_compat_early
     t = Time.xmlschema("1954-01-05T00:00:00.123456789+00:00")
-    json = Oj.dump(t, :mode => :compat, :second_precision => 5)
+    json = Oj.dump(t, :mode => :compat, :second_precision => 5, :time_format => :unix)
     assert_equal(%{-504575999.87654}, json)
   end
   def test_unix_time_compat_1970
     t = Time.xmlschema("1970-01-01T00:00:00.123456789+00:00")
-    json = Oj.dump(t, :mode => :compat, :second_precision => 5)
+    json = Oj.dump(t, :mode => :compat, :second_precision => 5, :time_format => :unix)
     assert_equal(%{0.12346}, json)
   end
   def test_ruby_time_compat
@@ -398,11 +460,7 @@ class Juice < Minitest::Test
         sign = '-'
         tz = -tz
       end
-      if RUBY_VERSION.start_with?('1.8')
-        assert_equal(%{"2012-01-05T23:58:07%sZ"} % [sign], json)
-      else
-        assert_equal(%{"2012-01-05T23:58:07%s%02d:%02d"} % [sign, tz / 3600, tz / 60 % 60], json)
-      end
+      assert_equal(%{"2012-01-05T23:58:07%s%02d:%02d"} % [sign, tz / 3600, tz / 60 % 60], json)
     end
   end
   def test_xml_time_compat_precision
@@ -440,11 +498,7 @@ class Juice < Minitest::Test
         sign = '-'
         tz = -tz
       end
-      if RUBY_VERSION.start_with?('1.8')
-        assert_equal(%{"2012-01-05T23:58:08%sZ"} % [sign], json)
-      else
-        assert_equal(%{"2012-01-05T23:58:08%s%02d:%02d"} % [sign, tz / 3600, tz / 60 % 60], json)
-      end
+      assert_equal(%{"2012-01-05T23:58:08%s%02d:%02d"} % [sign, tz / 3600, tz / 60 % 60], json)
     end
   end
   def test_xml_time_compat_zulu
@@ -460,16 +514,6 @@ class Juice < Minitest::Test
       assert_equal(%{"2012-01-05T23:58:07Z"}, json)
     end
   end
-  def test_time_object
-    t = Time.now()
-    Oj.default_options = { :mode => :object }
-    dump_and_load(t, false)
-  end
-  def test_time_object_early
-    t = Time.xmlschema("1954-01-05T00:00:00.123456")
-    Oj.default_options = { :mode => :object }
-    dump_and_load(t, false)
-  end
 
   # Class
   def test_class_strict
@@ -494,6 +538,29 @@ class Juice < Minitest::Test
     dump_and_load(Juice, false)
   end
 
+  # Module
+  def test_module_strict
+    begin
+      Oj.dump(TestModule, :mode => :strict)
+    rescue Exception
+      assert(true)
+      return
+    end
+    assert(false, "*** expected an exception")
+  end
+  def test_module_null
+    json = Oj.dump(TestModule, :mode => :null)
+    assert_equal('null', json)
+  end
+  def test_module_compat
+    json = Oj.dump(TestModule, :mode => :compat)
+    assert_equal(%{"Juice::TestModule"}, json)
+  end
+  def test_module_object
+    Oj.default_options = { :mode => :object }
+    dump_and_load(TestModule, false)
+  end
+
   # Hash
   def test_hash
     Oj.default_options = { :mode => :strict }
@@ -817,8 +884,6 @@ class Juice < Minitest::Test
       json = Oj.dump(1..7, :mode => :object, :indent => 0)
       if 'rubinius' == $ruby
         assert(%{{"^O":"Range","begin":1,"end":7,"exclude_end?":false}} == json)
-      elsif 'jruby' == $ruby
-        assert(%{{"^O":"Range","begin":1,"end":7,"exclude_end?":false}} == json)
       else
         assert_equal(%{{"^u":["Range",1,7,false]}}, json)
       end
@@ -888,7 +953,7 @@ class Juice < Minitest::Test
     BigDecimal.send(:define_method, :as_json) do
       %{this is big}
     end
-    json = Oj.dump(orig, :mode => :compat)
+    json = Oj.dump(orig, :mode => :compat, :bigdecimal_as_decimal => false)
     bg = Oj.load(json, :mode => :compat)
     assert_equal("this is big", bg)
     BigDecimal.send(:remove_method, :as_json) # cleanup
@@ -914,12 +979,8 @@ class Juice < Minitest::Test
     rescue Oj::ParseError
       assert(true)
     end
-    begin
-      Oj.load('Infinity', :mode => :compat)
-      fail()
-    rescue Oj::ParseError
-      assert(true)
-    end
+    x = Oj.load('Infinity', :mode => :compat)
+    assert_equal('Infinity', x.to_s)
 
   end
 
@@ -1054,7 +1115,6 @@ class Juice < Minitest::Test
 
 # Stream Deeply Nested
   def test_deep_nest
-    #unless 'jruby' == RUBY_DESCRIPTION.split(' ')[0]
     begin
       n = 10000
       Oj.strict_load('[' * n + ']' * n)
